#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <wolfssl/ssl.h>
#include <wolfssl/wolfcrypt/error-crypt.h>
#include <wolfssl/openssl/err.h>
#include <unistd.h>
#include <arpa/inet.h>

#define CERT_FILE "../certificates/ca-cert.pem"
#define SERVER_IP "192.168.178.51" // Die IP-Adresse des Servers
#define PORT 5000
#define BUFFER_SIZE 1024

void initWolfSSL() {
    wolfSSL_Init();  // Initialize WolfSSL library
}

void cleanupWolfSSL() {
    wolfSSL_Cleanup();  // Cleanup WolfSSL library
}

int createClientSocket(const char *server_ip, int port) {
    int client_fd;
    struct sockaddr_in server_addr;

    if ((client_fd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
        perror("Socket creation failed");
        exit(EXIT_FAILURE);
    }

    memset(&server_addr, 0, sizeof(server_addr));
    server_addr.sin_family = AF_INET;
    server_addr.sin_port = htons(port);
    if (inet_pton(AF_INET, server_ip, &server_addr.sin_addr) <= 0) {
        perror("Invalid server IP address");
        close(client_fd);
        exit(EXIT_FAILURE);
    }

    if (connect(client_fd, (struct sockaddr *)&server_addr, sizeof(server_addr)) == -1) {
        perror("Connection failed");
        close(client_fd);
        exit(EXIT_FAILURE);
    }

    return client_fd;
}

int main() {
    // Verschlüsselungsalgorithmus auswählen (AES, RSA, etc.)
    int choice;
    printf("Wählen Sie den Verschlüsselungsalgorithmus:\n");
    printf("1. TLSv1.2 mit AES\n");
    printf("2. TLSv1.3\n");
    printf("Geben Sie Ihre Wahl ein (1 oder 2): ");
    scanf("%d", &choice);

    // Initialize WolfSSL
    initWolfSSL();

    // Konfiguration des Verschlüsselungsalgorithmus
    const WOLFSSL_METHOD *method;  // Korrekte Methode für WolfSSL
    if (choice == 1) {
        method = wolfTLSv1_2_client_method(); // TLSv1.2
    } else {
        method = wolfTLSv1_3_client_method(); // TLSv1.3
    }

    // Load CA certificate
    WOLFSSL_CTX *ctx = wolfSSL_CTX_new(method);
    if (ctx == NULL) {
        printf("Failed to create SSL context\n");
        cleanupWolfSSL();
        exit(EXIT_FAILURE);
    }

    if (wolfSSL_CTX_load_verify_locations(ctx, CERT_FILE, NULL) != SSL_SUCCESS) {
        printf("Failed to load CA certificate\n");
        wolfSSL_CTX_free(ctx);
        cleanupWolfSSL();
        exit(EXIT_FAILURE);
    }

    // Create client socket
    int client_fd = createClientSocket(SERVER_IP, PORT);

    // Create SSL object
    WOLFSSL *ssl = wolfSSL_new(ctx);
    if (ssl == NULL) {
        printf("Failed to create SSL object\n");
        close(client_fd);
        wolfSSL_CTX_free(ctx);
        cleanupWolfSSL();
        exit(EXIT_FAILURE);
    }

    // Associate socket with SSL object
    if (wolfSSL_set_fd(ssl, client_fd) != SSL_SUCCESS) {
        printf("Failed to associate socket with SSL object\n");
        wolfSSL_free(ssl);
        close(client_fd);
        wolfSSL_CTX_free(ctx);
        cleanupWolfSSL();
        exit(EXIT_FAILURE);
    }

    // Perform TLS handshake
    if (wolfSSL_connect(ssl) != SSL_SUCCESS) {
        printf("SSL handshake failed\n");
        wolfSSL_free(ssl);
        close(client_fd);
        wolfSSL_CTX_free(ctx);
        cleanupWolfSSL();
        exit(EXIT_FAILURE);
    }

    printf("Connected to the server!\n");

    // Open the file to send
    const char *file_path = "../testData/test.txt";
    FILE *file = fopen(file_path, "rb");
    if (file == NULL) {
        perror("Failed to open file");
        wolfSSL_free(ssl);
        close(client_fd);
        wolfSSL_CTX_free(ctx);
        cleanupWolfSSL();
        exit(EXIT_FAILURE);
    }

    // Send file in chunks
    char buffer[BUFFER_SIZE];
    size_t bytes_read;
    while ((bytes_read = fread(buffer, 1, sizeof(buffer), file)) > 0) {
        if (wolfSSL_write(ssl, buffer, bytes_read) != bytes_read) {
            printf("Error sending file\n");
            fclose(file);
            wolfSSL_free(ssl);
            close(client_fd);
            wolfSSL_CTX_free(ctx);
            cleanupWolfSSL();
            exit(EXIT_FAILURE);
        }
    }

    printf("File '%s' successfully sent to the server.\n", file_path);

    // Clean up
    fclose(file);
    wolfSSL_free(ssl);
    close(client_fd);
    wolfSSL_CTX_free(ctx);
    cleanupWolfSSL();

    return 0;
}
