#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <wolfssl/ssl.h>
#include <wolfssl/wolfcrypt/error-crypt.h>
#include <wolfssl/openssl/err.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>

#define CERT_FILE "../certificates/server-cert.pem"
#define KEY_FILE "../certificates/server-key.pem"
#define PORT 5000
#define BUFFER_SIZE 1024

void initWolfSSL() {
    wolfSSL_Init();  // Initialize WolfSSL library
}

void cleanupWolfSSL() {
    wolfSSL_Cleanup();  // Cleanup WolfSSL library
}

int createServerSocket(int port) {
    int server_fd;
    struct sockaddr_in server_addr;

    if ((server_fd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
        perror("Socket creation failed");
        exit(EXIT_FAILURE);
    }

    memset(&server_addr, 0, sizeof(server_addr));
    server_addr.sin_family = AF_INET;
    server_addr.sin_addr.s_addr = INADDR_ANY;
    server_addr.sin_port = htons(port);

    if (bind(server_fd, (struct sockaddr *)&server_addr, sizeof(server_addr)) == -1) {
        perror("Binding failed");
        close(server_fd);
        exit(EXIT_FAILURE);
    }

    if (listen(server_fd, 1) == -1) {
        perror("Listening failed");
        close(server_fd);
        exit(EXIT_FAILURE);
    }

    printf("Server is listening on port %d...\n", port);
    return server_fd;
}

int main() {
    // Verschlüsselungsalgorithmus auswählen (AES, RSA, etc.)
    int choice;
    printf("Wählen Sie den Verschlüsselungsalgorithmus:\n");
    printf("1. TLSv1.2 mit AES\n");
    printf("2. TLSv1.3\n");
    printf("Geben Sie Ihre Wahl ein (1 oder 2): ");
    scanf("%d", &choice);

    // Initialize WolfSSL
    initWolfSSL();

    // Create server socket
    int server_fd = createServerSocket(PORT);

    // Konfiguration des Verschlüsselungsalgorithmus
    const WOLFSSL_METHOD *method;  // Korrekte Methode für WolfSSL
    if (choice == 1) {
        method = wolfTLSv1_2_client_method(); // TLSv1.2
    } else {
        method = wolfTLSv1_3_client_method(); // TLSv1.3
    }

    // SSL-Kontext erstellen
    WOLFSSL_CTX *ctx = wolfSSL_CTX_new(method);
    if (ctx == NULL) {
        printf("Failed to create SSL context\n");
        cleanupWolfSSL();
        exit(EXIT_FAILURE);
    }

    if (wolfSSL_CTX_use_certificate_file(ctx, CERT_FILE, SSL_FILETYPE_PEM) != SSL_SUCCESS) {
        printf("Failed to load certificate\n");
        wolfSSL_CTX_free(ctx);
        cleanupWolfSSL();
        exit(EXIT_FAILURE);
    }

    if (wolfSSL_CTX_use_PrivateKey_file(ctx, KEY_FILE, SSL_FILETYPE_PEM) != SSL_SUCCESS) {
        printf("Failed to load private key\n");
        wolfSSL_CTX_free(ctx);
        cleanupWolfSSL();
        exit(EXIT_FAILURE);
    }

    // Accept client connection
    int client_fd = accept(server_fd, NULL, NULL);
    if (client_fd == -1) {
        perror("Accept failed");
        close(server_fd);
        wolfSSL_CTX_free(ctx);
        cleanupWolfSSL();
        exit(EXIT_FAILURE);
    }

    // Create SSL object
    WOLFSSL *ssl = wolfSSL_new(ctx);
    if (ssl == NULL) {
        printf("Failed to create SSL object\n");
        close(client_fd);
        close(server_fd);
        wolfSSL_CTX_free(ctx);
        cleanupWolfSSL();
        exit(EXIT_FAILURE);
    }

    // Associate socket with SSL object
    if (wolfSSL_set_fd(ssl, client_fd) != SSL_SUCCESS) {
        printf("Failed to associate socket with SSL object\n");
        wolfSSL_free(ssl);
        close(client_fd);
        close(server_fd);
        wolfSSL_CTX_free(ctx);
        cleanupWolfSSL();
        exit(EXIT_FAILURE);
    }

    // Perform TLS handshake
    if (wolfSSL_accept(ssl) != SSL_SUCCESS) {
        printf("SSL handshake failed\n");
        wolfSSL_free(ssl);
        close(client_fd);
        close(server_fd);
        wolfSSL_CTX_free(ctx);
        cleanupWolfSSL();
        exit(EXIT_FAILURE);
    }

    printf("TLS handshake completed successfully\n");

    // Receive file data
    FILE *file = fopen("received_file.txt", "wb");
    if (file == NULL) {
        perror("File open failed");
        wolfSSL_free(ssl);
        close(client_fd);
        close(server_fd);
        wolfSSL_CTX_free(ctx);
        cleanupWolfSSL();
        exit(EXIT_FAILURE);
    }

    char buffer[BUFFER_SIZE];
    int bytes_received;
    while ((bytes_received = wolfSSL_read(ssl, buffer, sizeof(buffer))) > 0) {
        fwrite(buffer, 1, bytes_received, file);
    }

    if (bytes_received < 0) {
        printf("Error receiving file\n");
    } else {
        printf("File successfully received and saved as 'received_file.txt'\n");
    }

    fclose(file);

    // Clean up
    wolfSSL_free(ssl);
    close(client_fd);
    close(server_fd);
    wolfSSL_CTX_free(ctx);
    cleanupWolfSSL();

    return 0;
}
